<!DOCTYPE html>
<html lang="en">

<head>
    <title>{{ title }}</title>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="stylesheet" type="text/css" href="//cdnjs.cloudflare.com/ajax/libs/swagger-ui/3.30.0/swagger-ui.css" />
        <style>
            html
              {
                box-sizing: border-box;
                overflow: -moz-scrollbars-vertical;
                overflow-y: scroll;
              }

              *,
              *:before,
              *:after
              {
                box-sizing: inherit;
              }

              body
              {
                margin:0;
                background: #fafafa;
              }
        </style>
</head>

<body>
    <div id="swagger-ui"></div>
    <script src="//cdnjs.cloudflare.com/ajax/libs/swagger-ui/3.30.0/swagger-ui-bundle.js"></script>
    <script src="//cdnjs.cloudflare.com/ajax/libs/swagger-ui/3.30.0/swagger-ui-standalone-preset.js"></script>

    <script>
        window.onload = function() {

          // Begin Swagger UI call region
          const ui = SwaggerUIBundle({
            "dom_id": "#swagger-ui",
            deepLinking: true,
            presets: [
              SwaggerUIBundle.presets.apis,
              SwaggerUIStandalonePreset
            ],
            plugins: [
              SwaggerUIBundle.plugins.DownloadUrl
            ],
            layout: "StandaloneLayout",
            validatorUrl: "https://validator.swagger.io/validator",
            url: "http://localhost:8000/api/rest/kitsune-schema/",
          })


          // End Swagger UI call region


          window.ui = ui
        }
    </script>
</body>

</html>
